@@ -3,6 +3,7 @@
     SPDX-FileCopyrightText: 2005 Olivier Goffart <ogoffart at kde.org>
     SPDX-FileCopyrightText: 2013-2015 Martin Klapetek <mklapetek@kde.org>
     SPDX-FileCopyrightText: 2017 Eike Hein <hein@kde.org>
+    SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
 
     SPDX-License-Identifier: LGPL-2.0-only
 */
@@ -13,7 +14,6 @@
 
 #include <config-knotifications.h>
 
-#include <KPluginMetaData>
 #include <QFileInfo>
 #include <QHash>
 
@@ -26,10 +26,6 @@
 #include "knotificationreplyaction.h"
 #include "knotifyconfig.h"
 
-#include "notifybyexecute.h"
-#include "notifybylogfile.h"
-#include "notifybytaskbar.h"
-
 #if defined(Q_OS_ANDROID)
 #include "notifybyandroid.h"
 #elif defined(Q_OS_MACOS)
@@ -43,13 +39,7 @@
 #include "debug_p.h"
 
 #if defined(HAVE_CANBERRA)
-#include "notifybyaudio_canberra.h"
-#elif defined(HAVE_PHONON4QT5)
-#include "notifybyaudio_phonon.h"
-#endif
-
-#ifdef HAVE_SPEECH
-#include "notifybytts.h"
+#include "notifybyaudio.h"
 #endif
 
 typedef QHash<QString, QString> Dict;
@@ -82,9 +72,7 @@ KNotificationManager::KNotificationManager()
     d->notifyPlugins.clear();
 
 #ifdef QT_DBUS_LIB
-    const bool inSandbox = QFileInfo::exists(QLatin1String("/.flatpak-info")) || qEnvironmentVariableIsSet("SNAP");
-
-    if (inSandbox) {
+    if (isInsideSandbox()) {
         QDBusConnectionInterface *interface = QDBusConnection::sessionBus().interface();
         d->portalDBusServiceExists = interface->isServiceRegistered(QStringLiteral("org.freedesktop.portal.Desktop"));
     }
@@ -98,10 +86,7 @@ KNotificationManager::KNotificationManager()
 #endif
 }
 
-KNotificationManager::~KNotificationManager()
-{
-    delete d;
-}
+KNotificationManager::~KNotificationManager() = default;
 
 KNotificationPlugin *KNotificationManager::pluginForAction(const QString &action)
 {
@@ -138,89 +123,11 @@ KNotificationPlugin *KNotificationManager::pluginForAction(const QString &action
         }
 #endif
         addPlugin(plugin);
-    } else if (action == QLatin1String("Taskbar")) {
-#if !defined(Q_OS_ANDROID)
-        plugin = new NotifyByTaskbar(this);
-        addPlugin(plugin);
-#endif
     } else if (action == QLatin1String("Sound")) {
-#if defined(HAVE_PHONON4QT5) || defined(HAVE_CANBERRA)
+#if defined(HAVE_CANBERRA)
         plugin = new NotifyByAudio(this);
         addPlugin(plugin);
 #endif
-    } else if (action == QLatin1String("Execute")) {
-#if !defined(Q_OS_ANDROID)
-        plugin = new NotifyByExecute(this);
-        addPlugin(plugin);
-#endif
-    } else if (action == QLatin1String("Logfile")) {
-        plugin = new NotifyByLogfile(this);
-        addPlugin(plugin);
-    } else if (action == QLatin1String("TTS")) {
-#ifdef HAVE_SPEECH
-        plugin = new NotifyByTTS(this);
-        addPlugin(plugin);
-#endif
-    } else {
-        bool pluginFound = false;
-
-        std::function<bool(const KPluginMetaData &)> filter = [&action, &pluginFound](const KPluginMetaData &data) {
-            // KPluginMetaData::findPlugins loops over the plugins it
-            // finds and calls this function to determine whether to
-            // deliver them. We use a `pluginFound` var outside the
-            // lambda to break out of the loop once we got a match.
-            // The reason we can't just have KPluginMetaData::findPlugins,
-            // loop over the meta data and instantiate only one plugin
-            // is because the X-KDE-KNotification-OptionName field is
-            // optional (see TODO note below) and the matching plugin
-            // may be among the plugins which don't have it.
-            if (pluginFound) {
-                return false;
-            }
-
-            const QJsonObject &rawData = data.rawData();
-
-            // This field is new-ish and optional. If it's not set we always
-            // instantiate the plugin, unless we already got a match.
-            // TODO KF6: Require X-KDE-KNotification-OptionName be set and
-            // reject plugins without it.
-            if (rawData.contains(QLatin1String("X-KDE-KNotification-OptionName"))) {
-                if (rawData.value(QStringLiteral("X-KDE-KNotification-OptionName")) == action) {
-                    pluginFound = true;
-                } else {
-                    return false;
-                }
-            }
-
-            return true;
-        };
-
-        QPluginLoader loader;
-        const QVector<KPluginMetaData> listMetaData = KPluginMetaData::findPlugins(QStringLiteral("knotification/notifyplugins"), filter);
-        for (const KPluginMetaData &metadata : listMetaData) {
-            loader.setFileName(metadata.fileName());
-            QObject *pluginObj = loader.instance();
-            if (!pluginObj) {
-                qCWarning(LOG_KNOTIFICATIONS).nospace() << "Could not instantiate plugin \"" << metadata.fileName() << "\": " << loader.errorString();
-                continue;
-            }
-            KNotificationPlugin *notifyPlugin = qobject_cast<KNotificationPlugin *>(pluginObj);
-
-            if (notifyPlugin) {
-                notifyPlugin->setParent(this);
-                // We try to avoid unnecessary instantiations (see above), but
-                // when they happen keep the resulting plugins around.
-                addPlugin(notifyPlugin);
-
-                // Get ready to return the plugin we got asked for.
-                if (notifyPlugin->optionName() == action) {
-                    plugin = notifyPlugin;
-                }
-            } else {
-                // Not our/valid plugin, so delete the created object.
-                pluginObj->deleteLater();
-            }
-        }
     }
 
     return plugin;
@@ -235,12 +142,12 @@ void KNotificationManager::notifyPluginFinished(KNotification *notification)
     notification->deref();
 }
 
-void KNotificationManager::notificationActivated(int id, int action)
+void KNotificationManager::notificationActivated(int id, const QString &actionId)
 {
     if (d->notifications.contains(id)) {
-        qCDebug(LOG_KNOTIFICATIONS) << id << " " << action;
+        qCDebug(LOG_KNOTIFICATIONS) << id << " " << actionId;
         KNotification *n = d->notifications[id];
-        n->activate(action);
+        n->activate(actionId);
 
         // Resident actions delegate control over notification lifetime to the client
         if (!n->hints().value(QStringLiteral("resident")).toBool()) {
@@ -291,9 +198,9 @@ void KNotificationManager::notificationClosed()
     }
 }
 
-void KNotificationManager::close(int id, bool force)
+void KNotificationManager::close(int id)
 {
-    if (force || d->notifications.contains(id)) {
+    if (d->notifications.contains(id)) {
         KNotification *n = d->notifications.value(id);
         qCDebug(LOG_KNOTIFICATIONS) << "Closing notification" << id;
 
@@ -304,7 +211,7 @@ void KNotificationManager::close(int id, bool force)
         // For example: Action=Popup is a single actions but there is 5 loaded
         // plugins, calling close() on the second would already close-and-delete
         // the notification
-        KNotifyConfig notifyConfig(n->appName(), n->contexts(), n->eventId());
+        KNotifyConfig notifyConfig(n->appName(), n->eventId());
         QString notifyActions = notifyConfig.readEntry(QStringLiteral("Action"));
 
         const auto listActions = notifyActions.split(QLatin1Char('|'));
@@ -321,13 +228,17 @@ void KNotificationManager::close(int id, bool force)
 
 void KNotificationManager::notify(KNotification *n)
 {
-    KNotifyConfig notifyConfig(n->appName(), n->contexts(), n->eventId());
+    KNotifyConfig notifyConfig(n->appName(), n->eventId());
 
     if (d->dirtyConfigCache.contains(n->appName())) {
         notifyConfig.reparseSingleConfiguration(n->appName());
         d->dirtyConfigCache.removeOne(n->appName());
     }
 
+    if (!notifyConfig.isValid()) {
+        qCWarning(LOG_KNOTIFICATIONS) << "No event config could be found for event id" << n->eventId() << "under notifyrc file for app" << n->appName();
+    }
+
     const QString notifyActions = notifyConfig.readEntry(QStringLiteral("Action"));
 
     if (notifyActions.isEmpty() || notifyActions == QLatin1String("None")) {
@@ -377,7 +288,7 @@ void KNotificationManager::notify(KNotification *n)
         }
 
         qCDebug(LOG_KNOTIFICATIONS) << "Calling notify on" << notifyPlugin->optionName();
-        notifyPlugin->notify(n, &notifyConfig);
+        notifyPlugin->notify(n, notifyConfig);
     }
 
     connect(n, &KNotification::closed, this, &KNotificationManager::notificationClosed);
@@ -385,10 +296,10 @@ void KNotificationManager::notify(KNotification *n)
 
 void KNotificationManager::update(KNotification *n)
 {
-    KNotifyConfig notifyConfig(n->appName(), n->contexts(), n->eventId());
+    KNotifyConfig notifyConfig(n->appName(), n->eventId());
 
     for (KNotificationPlugin *p : std::as_const(d->notifyPlugins)) {
-        p->update(n, &notifyConfig);
+        p->update(n, notifyConfig);
     }
 }
 
@@ -404,4 +315,13 @@ void KNotificationManager::reparseConfiguration(const QString &app)
     }
 }
 
+bool KNotificationManager::isInsideSandbox()
+{
+    // logic is taken from KSandbox::isInside()
+    static const bool isFlatpak = QFileInfo::exists(QStringLiteral("/.flatpak-info"));
+    static const bool isSnap = qEnvironmentVariableIsSet("SNAP");
+
+    return isFlatpak || isSnap;
+}
+
 #include "moc_knotificationmanager_p.cpp"
